@@ -1,8 +1,8 @@
-/* $Id: Browser.cpp 119 2008-02-18 15:23:07Z dezperado $ */
+/* $Id: Browser.cpp 272 2009-12-21 17:10:21Z dezperado $ */
 /*
  Browser.cpp : Fim image browser
 
- (c) 2007-2008 Michele Martone
+ (c) 2007-2009 Michele Martone
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,15 +20,14 @@
 */
 
 #define firstorzero(x) (x.size()?((int)(x[0])):0)
+#define firstforzero(x) (x.size()?((float)(x[0])):0.0)
 
 #include <dirent.h>
-#include <sys/types.h>
+#include <sys/types.h>	/* POSIX Standard: 2.6 Primitive System Data Types (e.g.: ssize_t) */
 #include "fim.h"
 
 namespace fim
 {
-	extern fim::CommandConsole cc;
-	
 	int Browser::current_n()const{ return current_n(cp); }
 	int Browser::current_n(int ccp)const{ return ccp?ccp-1:ccp; }
 
@@ -38,8 +37,7 @@ namespace fim
 		 * returns a string with the info about the files in list
 		 */
 		fim::string fileslist;
-		for(unsigned int i=0;i<flist.size();++i)fileslist+=flist[i]+fim::string(" ");
-		cout << fileslist.c_str();
+		for(size_t i=0;i<flist.size();++i)fileslist+=flist[i]+fim::string(" ");
 		return fileslist;
 	}
 
@@ -48,7 +46,7 @@ namespace fim
 		/*
 		 * accessory method
 		 */
-		for(unsigned int i=0;i<flist.size();++i)
+		for(size_t i=0;i<flist.size();++i)
 			os << flist[i] << "\n";
 		return os;
 	}
@@ -83,7 +81,7 @@ namespace fim
 				 * viewport().redisplay(); <- and this should be good
 				 * :)
 				 * */
-				if( cc.redisplay() )
+				if( commandConsole.redisplay() )
 					this->display_status(current().c_str(), NULL);
 			}
 #ifdef FIM_AUTOCMDS
@@ -92,16 +90,24 @@ namespace fim
 		}
 	}
 
-	Browser::Browser():nofile("")
+#ifdef FIM_READ_STDIN_IMAGE
+	void Browser::set_default_image(Image *stdin_image)
+	{
+		/*
+		 * this is used mainly to set image files read from pipe or stdin
+		 * */
+		if(!stdin_image || stdin_image->check_invalid())return;
+
+		if(default_image) delete default_image;
+		default_image=stdin_image;
+	}
+#endif
+
+	Browser::Browser(CommandConsole &cc):nofile(""),commandConsole(cc)
 	{	
 		/*
 		 * we initialize to no file the current file name
 		 */
-#ifndef FIM_WINDOWS
-		only_viewport = new Viewport();
-		// DANGER !!
-		// EXCEPTIONS NEEDED
-#endif
 		cp=0;	//and to file index 0 (no file)
 	}
 
@@ -117,11 +123,11 @@ namespace fim
 		if(flist.size()<=0)return nofile;
 		assert(cp);
 		flist.erase(flist.begin()+current_n());
-		setGlobalVariable("filelistlen",current_images());
+		setGlobalVariable(FIM_VID_FILELISTLEN,current_images());
 		return s;
 	}
 
-	const fim::string Browser::pop()
+	const fim::string Browser::pop(fim::string filename)
 	{	
 		/*
 		 * pops the last image filename from the filenames list
@@ -130,10 +136,19 @@ namespace fim
 		fim::string s;
 		if(flist.size()<=0)return nofile;
 		assert(cp);
-		if(current_n()==(int)flist.size())cp--;
-		s=flist[flist.size()-1];
-		flist.erase(flist.begin()+current_n());
-		setGlobalVariable("filelistlen",current_images());
+		if(filename=="")
+		{
+			if(current_n()==(int)flist.size())cp--;
+			s=flist[flist.size()-1];
+			flist.erase(flist.begin()+current_n());
+		}
+		else
+		{
+			for(size_t i=0;i<flist.size();++i)
+				if(flist[i]==filename)
+					flist.erase(flist.begin()+i);
+		}
+		setGlobalVariable(FIM_VID_FILELISTLEN,current_images());
 		return s;
 	}
 
@@ -298,7 +313,7 @@ namespace fim
 		 */
 		double multiscale;
 		if(args.size()==0)return "";
-		multiscale=atof(args[0].c_str());
+		multiscale=fim_atof(args[0].c_str());
 		if(multiscale==0.0)return "";
 		if(c_image())
 		{
@@ -321,7 +336,7 @@ namespace fim
 		 */
 		double deltascale;
 		if(args.size()==0)return "";
-		deltascale=atof(args[0].c_str());
+		deltascale=fim_atof(args[0].c_str());
 		if(deltascale==0.0)return "";
 		if(strstr(args[0].c_str(),"%"))deltascale*=.01;
 		if(c_image())
@@ -345,7 +360,7 @@ namespace fim
 		 */
 		double newscale;
 		if(args.size()==0)return "";
-		newscale=atof(args[0].c_str());
+		newscale=fim_atof(args[0].c_str());
 		if(newscale==0.0)return "";
 		if(strstr(args[0].c_str(),"%"))newscale*=.01;
 		if(c_image())
@@ -459,13 +474,24 @@ namespace fim
 		return "";
 	}
 
+	fim::string Browser::negate(const args_t &args)
+	{
+		/*
+		 */
+		if(!image() )
+			return "";
+
+		if(image() && image()->negate())
+			return "";
+	}
+
 	fim::string Browser::display_status(const char *l,const char *r)
 	{
 		/*
 		 * displays the left text message and a right bracketed one
 		 */
-		if(getGlobalIntVariable("_display_status"))
-			cc.set_status_bar((const char*)l, image()?(image()->getInfo().c_str()):"*");
+		if(getGlobalIntVariable(FIM_VID_DISPLAY_STATUS))
+			commandConsole.set_status_bar((const char*)l, image()?(image()->getInfo().c_str()):"*");
 		return "";
 	}
 
@@ -483,7 +509,7 @@ namespace fim
 			/*
 			 * the following is a trick to override redisplaying..
 			 */
-			if(image() && getGlobalIntVariable("_override_display")==0)
+			if(image() && getGlobalIntVariable(FIM_VID_OVERRIDE_DISPLAY)==0)
 			//	if(c_image())
 			{
 				//fb_clear_screen();
@@ -491,17 +517,17 @@ namespace fim
 				/*
 				 * we redraw the whole screen and thus all of the windows
 				 * */
-				if( cc.display() )
+				if( commandConsole.display() )
 					this->display_status(current().c_str(), NULL);
 //				FIXME:
-//				if(cc.window)cc.window->recursive_display();
+//				if(commandConsole.window)commandConsole.window->recursive_display();
 			}
 #ifdef FIM_AUTOCMDS
 			autocmd_exec("PostDisplay",c);
 #endif
 		}
 		else{ cout << "no image to display, sorry!";
-		cc.set_status_bar("no image loaded.", "*");}
+		commandConsole.set_status_bar("no image loaded.", "*");}
 		return "";
 	}
 
@@ -520,23 +546,29 @@ namespace fim
 		 * assume there was a load attempt : check and take some action in case of error
 		 *
 		 * FIXME : this behaviour is BUGGY, because recursion will be killed off 
-		 *         by the autocommand loop prevention mechanism.
+		 *         by the autocommand loop prevention mechanism. (this is not true, as 20090215)
 		 * */
-		if(image() && viewport() && ! (viewport()->check_valid()))
+		static int lehsof=0;	/* './fim FILE NONFILE' and hitting 'prev' will make this necessary  */
+
+		if(lehsof)return 0; /* this prevents infinite recursion */
+		if(/*image() &&*/ viewport() && ! (viewport()->check_valid()))
 		{
 			free_current_image();
+			++lehsof;
 #ifdef FIM_REMOVE_FAILED
-			if(current()!="")
-			{
-				pop_current();	//removes the current file from the list.
+				//pop(c);	//removes the currently specified file from the list. (pop doesn't work in this way)
+				args_t args;
+				args.push_back(c.c_str());
+				remove(args);	// remove is an experimental function
 #ifdef FIM_AUTOSKIP_FAILED
-#ifdef FIM_ALWAYS_UNDEFINED
-				if(n_files())next(1);
-				reload(); /* this will not be effective ! */
+				if(n_files())
+				{
+					//next(1);
+					reload(); /* this is effective, at least partially */
+				}
 #endif
 #endif
-			}
-#endif
+			--lehsof;
 			return 1;
 		}
 		return 0;
@@ -564,10 +596,29 @@ namespace fim
 		try
 		{
 #ifndef FIM_BUGGED_CACHE
-		if(viewport())viewport()->setImage( cache.useCachedImage(current().c_str()) );
+	#ifdef FIM_CACHE_DEBUG
+		if(viewport())std::cout << "browser::loadCurrentImage(\"" << current().c_str() << "\")\n";
+	#endif
+		if(viewport())viewport()->setImage( cache.useCachedImage(cache_key_t(current(),(current()==FIM_STDIN_IMAGE_NAME)?FIM_E_STDIN:FIM_E_FILE)) );// FIXME
 #else
 		// warning : in this cases exception handling is missing
+	#ifdef FIM_READ_STDIN_IMAGE
+		if(current()!=FIM_STDIN_IMAGE_NAME)
+		{
+			if(viewport())viewport()->setImage( new Image(current().c_str()) );
+		}
+		else
+		{
+			if( viewport() && default_image )
+			{
+				// a one time only image (new, experimental)
+				viewport()->setImage(default_image->getClone());
+				//default_image=NULL;
+			}
+		}
+	#else
 		if(viewport())viewport()->setImage( new Image(current().c_str()) );
+	#endif
 #endif
 		}
 		catch(FimException e)
@@ -586,7 +637,7 @@ namespace fim
 		 * only cleans up the internal data structures
 		 * */
 		if(viewport())viewport()->free();
-		setGlobalVariable("_cache_status",cache.getReport().c_str());
+		setGlobalVariable(FIM_VID_CACHE_STATUS,cache.getReport().c_str());
 	}
 
 	fim::string Browser::prefetch(const args_t &args)
@@ -600,8 +651,28 @@ namespace fim
 		return " prefetching disabled";
 #endif
 
+#ifdef FIM_AUTOCMDS
+			autocmd_exec("PrePrefetch",current());
+#endif
 		if( args.size() > 0 )return "";
-		cache.prefetch(get_next_filename(1).c_str());
+
+		setGlobalVariable(FIM_VID_WANT_PREFETCH,0);
+		if(cache.prefetch(cache_key_t(get_next_filename( 1).c_str(),FIM_E_FILE)))// we prefetch 1 file forward
+#ifdef FIM_AUTOSKIP_FAILED
+			pop(get_next_filename( 1));/* if the filename doesn't match a loadable image, we remove it */
+#else
+			{}	/* beware that this could be dangerous and trigger loops */
+#endif
+		if(cache.prefetch(cache_key_t(get_next_filename(-1).c_str(),FIM_E_FILE)))// we prefetch 1 file backward
+#ifdef FIM_AUTOSKIP_FAILED
+			pop(get_next_filename(-1));/* if the filename doesn't match a loadable image, we remove it */
+#else
+			{}	/* beware that this could be dangerous and trigger loops */
+#endif
+#ifdef FIM_AUTOCMDS
+			autocmd_exec("PostPrefetch",current());
+#endif
+		setGlobalVariable(FIM_VID_WANT_PREFETCH,1);
 		return "";
 	}
 
@@ -613,18 +684,17 @@ namespace fim
 		 * tries to load a new one from the current filename
 		 */
 		fim::string c=current();
+		//for(size_t i=0;i<args.size();++i) push(args[i]);
 		if(empty_file_list())return "sorry, no image to reload\n";
 #ifdef FIM_AUTOCMDS
 		autocmd_exec("PreReload",c);
 #endif
 		free_current_image();
-
 		loadCurrentImage();
+		//if(image())image()->reload();
 
-		if(getGlobalIntVariable("_prefetch")) prefetch(args_t());/*this will become an autocommand*/
-
-		while( n_files() && viewport() && ! (viewport()->check_valid() ) && load_error_handle(c) );
-
+//		while( n_files() && viewport() && ! (viewport()->check_valid() ) && load_error_handle(c) );
+		load_error_handle(c);
 #ifdef FIM_AUTOCMDS
 		autocmd_exec("PostReload",c);
 #endif
@@ -637,6 +707,7 @@ namespace fim
 		 * loads the current file, if not already loaded
 		 */
 		fim::string c=current();
+		//for(size_t i=0;i<args.size();++i) push(args[i]);
 		if(image() && ( image()->getName() == current()) )
 		{
 			return "image already loaded\n";		//warning
@@ -645,13 +716,11 @@ namespace fim
 #ifdef FIM_AUTOCMDS
 		autocmd_exec("PreLoad",c);
 #endif
-		cc.set_status_bar("please wait while loading...", "*");
+		commandConsole.set_status_bar("please wait while loading...", "*");
 
 		loadCurrentImage();
 
-		if(getGlobalIntVariable("_prefetch")) prefetch(args_t());
-
-//		if(image() && ! (viewport().check_valid()))return load_error_handle(c);
+		load_error_handle(c);
 #ifdef FIM_AUTOCMDS
 		autocmd_exec("PostLoad",c);
 #endif
@@ -674,7 +743,7 @@ namespace fim
 		/* 
 		 * returns whether the file nf is in the files list
 		 */
-		for(unsigned int i=0;i<flist.size();++i)
+		for(size_t i=0;i<flist.size();++i)
 			if(flist[i]==nf)return true;
 		return false;
 	}
@@ -721,15 +790,6 @@ namespace fim
 		}
 		return closedir(dir)!=NULL;
 	}
-
-	bool Browser::is_dir(fim::string nf)const
-	{
-		struct stat stat_s;
-		/*	if the directory doesn't exist, return */
-		if(-1==stat(nf.c_str(),&stat_s))return false;
-		if( ! S_ISDIR(stat_s.st_mode))return false;
-		return true;
-	}
 #endif
 
 	bool Browser::push(fim::string nf)
@@ -738,7 +798,8 @@ namespace fim
 		 * FIX ME:
 		 * are we sure we want no repetition!????
 		 * */
-		
+		if(nf!=FIM_STDIN_IMAGE_NAME)
+		{
 #ifdef FIM_CHECK_FILE_EXISTENCE
 		/*
 		 * skip adding the filename in the list if
@@ -757,17 +818,24 @@ namespace fim
 		if(  S_ISDIR(stat_s.st_mode))return push_dir(nf);
 #endif
 		/*	we want a regular file .. */
-		if(! S_ISREG(stat_s.st_mode))
+		if(
+			! S_ISREG(stat_s.st_mode) 
+#define FIM_READ_BLK_DEVICES 1
+#ifdef FIM_READ_BLK_DEVICES
+			&& ! S_ISBLK(stat_s.st_mode)  // NEW
+#endif
+		)
 		{
 			/*
 			 * i am not fully sure this is effective
 			 * */
 			nf+=" is not a regular file!";
-			cc.set_status_bar(nf.c_str(), "*");
+			commandConsole.set_status_bar(nf.c_str(), "*");
 			return false;
 		}
 #endif
 
+		}
 #ifdef FIM_CHECK_DUPLICATES
 		if(present(nf))
 		{
@@ -777,7 +845,8 @@ namespace fim
 		}
 #endif
 		flist.push_back(nf);
-		setGlobalVariable("filelistlen",current_images());
+
+		setGlobalVariable(FIM_VID_FILELISTLEN,current_images());
 		if(cp==0)++cp;
 		return false;
 	}
@@ -822,13 +891,13 @@ namespace fim
 		/*
 		 * goes to the next filename-matching file
 		 */
-		unsigned int i,j,c=current_n(),s=flist.size();
+		size_t i,j,c=current_n(),s=flist.size();
 		if( args.size() < 1 || s < 1 )return "";
 		for(j=0;j<s;++j)
 		{
 			last_regexp=args[0];
 			i=(j+c+1)%s;
-			if(cc.regexp_match(flist[i].c_str(),args[0].c_str()))
+			if(commandConsole.regexp_match(flist[i].c_str(),args[0].c_str()))
 			{	
 				fim::string c=current();
 #ifdef FIM_AUTOCMDS
@@ -837,15 +906,15 @@ namespace fim
 				goto_image(i+1);
 #ifdef FIM_AUTOCMDS
 				autocmd_exec("PostGoto",c);
-				if(!cc.inConsole())
-					cc.set_status_bar((current()+fim::string(" matches \"")+args[0]+fim::string("\"")).c_str(),NULL);
+				if(!commandConsole.inConsole())
+					commandConsole.set_status_bar((current()+fim::string(" matches \"")+args[0]+fim::string("\"")).c_str(),NULL);
 				return "";
 #endif
 			}
 		}
 		cout << "sorry, no filename matches \""<<args[0]<<"\"\n";
-		if(!cc.inConsole())
-			cc.set_status_bar((fim::string("sorry, no filename matches \"")+
+		if(!commandConsole.inConsole())
+			commandConsole.set_status_bar((fim::string("sorry, no filename matches \"")+
 						args[0]+
 						fim::string("\"")).c_str(),NULL);
 		
@@ -859,12 +928,19 @@ namespace fim
 		 */
 		int N=flist.size();
 		if(!N)return "";
+
+		if( N==1 && c_image() && c_image()->is_multipage())
+		{
+			image()->goto_page(n);
+			return N;
+		}
+
 		cp=n;
 		if(cp<0)cp=(cp%N)+N+1;//+1 added lately
 		if(cp>N) cp=1+(n%N);
 		if(!cp)++cp;
-		setGlobalVariable("fileindex",current_image());
-		setGlobalVariable("filename", current().c_str());
+		setGlobalVariable(FIM_VID_FILEINDEX,current_image());
+		setGlobalVariable(FIM_VID_FILENAME, current().c_str());
 		fim::string result = n_files()?(flist[current_n()]):nofile;
 		return result;
 	}
@@ -923,7 +999,6 @@ namespace fim
 		return flist[current_n(ccp)];
 	}
 
-
 	fim::string Browser::do_next(int n)
 	{
 		/*
@@ -937,8 +1012,8 @@ namespace fim
 		cp+=N;
 		cp%=N;
 		if(!cp)cp=N;
-		setGlobalVariable("fileindex",current_image());
-		setGlobalVariable("filename", current().c_str());
+		setGlobalVariable(FIM_VID_FILEINDEX,current_image());
+		setGlobalVariable(FIM_VID_FILENAME, current().c_str());
 		fim::string result = n_files()?(flist[current_n()]):nofile;
 		return "";
 	}
@@ -990,13 +1065,15 @@ namespace fim
 			 * if this software will have success, we will have indices here :)
 			 * sort(rlist.begin(),rlist.end());...
 			 */
-			for(unsigned int r=0;r<rlist.size();++r)
-			for(unsigned int i=0;i<flist.size();++i)
+			for(size_t r=0;r<rlist.size();++r)
+			for(size_t i=0;i<flist.size();++i)
 			if(flist[i]==rlist[r])
 			{
 //				std::cout << "removing" << flist[i]<<"\n";
 				flist.erase(flist.begin()+i);
 			}
+			cp=cp%(flist.size()+1);// new
+			setGlobalVariable(FIM_VID_FILELISTLEN,current_images());
 			return "";
 		}
 		else
@@ -1073,7 +1150,7 @@ namespace fim
 		 */
 #if 0
 		string fl;
-		for(unsigned int r=0;r<flist.size();++r)
+		for(size_t r=0;r<flist.size();++r)
 		{
 			fl+=flist[r];
 			fl+="\n";
@@ -1103,9 +1180,9 @@ namespace fim
 		/*
 		 *	ALIAS AND DELETE ME!
 		 */
-		float sfm=getGlobalFloatVariable("scale_factor_multiplier");if(sfm<=1.0f)sfm=1.1f;
-		setGlobalVariable("reduce_factor",getGlobalFloatVariable("reduce_factor")*sfm);
-		setGlobalVariable("magnify_factor",getGlobalFloatVariable("magnify_factor")*sfm);
+		float sfm=getGlobalFloatVariable(FIM_VID_SCALE_FACTOR_MULTIPLIER);if(sfm<=1.0f)sfm=1.1f;
+		setGlobalVariable(FIM_VID_REDUCE_FACTOR,getGlobalFloatVariable(FIM_VID_REDUCE_FACTOR)*sfm);
+		setGlobalVariable(FIM_VID_MAGNIFY_FACTOR,getGlobalFloatVariable(FIM_VID_MAGNIFY_FACTOR)*sfm);
 		return "";
 	}
 
@@ -1114,9 +1191,9 @@ namespace fim
 		/*
 		 *	ALIAS AND DELETE ME!
 		 */
-		float sfm=getGlobalFloatVariable("scale_factor_multiplier");if(sfm<=1.0f)sfm=1.1f;
-		setGlobalVariable("reduce_factor",getGlobalFloatVariable("reduce_factor")/sfm);
-		setGlobalVariable("magnify_factor",getGlobalFloatVariable("magnify_factor")/sfm);
+		float sfm=getGlobalFloatVariable(FIM_VID_SCALE_FACTOR_MULTIPLIER);if(sfm<=1.0f)sfm=1.1f;
+		setGlobalVariable(FIM_VID_REDUCE_FACTOR,getGlobalFloatVariable(FIM_VID_REDUCE_FACTOR)/sfm);
+		setGlobalVariable(FIM_VID_MAGNIFY_FACTOR,getGlobalFloatVariable(FIM_VID_MAGNIFY_FACTOR)/sfm);
 		return "";
 	}
 
@@ -1125,9 +1202,9 @@ namespace fim
 		/*
 		 *	ALIAS AND DELETE ME!
 		 */
-		float sfd=getGlobalFloatVariable("scale_factor_delta");if(sfd<=0.0f)sfd=0.1f;
-		setGlobalVariable("reduce_factor",getGlobalFloatVariable("reduce_factor")+sfd);
-		setGlobalVariable("magnify_factor",getGlobalFloatVariable("magnify_factor")+sfd);
+		float sfd=getGlobalFloatVariable(FIM_VID_SCALE_FACTOR_DELTA);if(sfd<=0.0f)sfd=0.1f;
+		setGlobalVariable(FIM_VID_REDUCE_FACTOR,getGlobalFloatVariable(FIM_VID_REDUCE_FACTOR)+sfd);
+		setGlobalVariable(FIM_VID_MAGNIFY_FACTOR,getGlobalFloatVariable(FIM_VID_MAGNIFY_FACTOR)+sfd);
 		return "";
 	}
 
@@ -1136,13 +1213,44 @@ namespace fim
 		/*
 		 *	ALIAS AND DELETE ME!
 		 */
-		float sfd=getGlobalFloatVariable("scale_factor_delta");if(sfd<=0.0f)sfd=0.1f;
-//		setGlobalVariable("scale_factor",getGlobalFloatVariable("scale_factor")/1.1f);
-		setGlobalVariable("reduce_factor",getGlobalFloatVariable("reduce_factor")-sfd);
-		setGlobalVariable("magnify_factor",getGlobalFloatVariable("magnify_factor")-sfd);
+		float sfd=getGlobalFloatVariable(FIM_VID_SCALE_FACTOR_DELTA);if(sfd<=0.0f)sfd=0.1f;
+		setGlobalVariable(FIM_VID_REDUCE_FACTOR,getGlobalFloatVariable(FIM_VID_REDUCE_FACTOR)-sfd);
+		setGlobalVariable(FIM_VID_MAGNIFY_FACTOR,getGlobalFloatVariable(FIM_VID_MAGNIFY_FACTOR)-sfd);
 		return "";
 	}
 
+	fim::string Browser::rotate(const args_t &args)
+	{
+		/*
+		 * rotates the displayed image a specified amount of degrees
+		 */ 
+		double angle;
+		if(args.size()==0)angle=1.0;
+		else angle=fim_atof(args[0].c_str());
+		if(angle==0.0)return "";
+
+		if(c_image())
+		{
+			//angle = (double)getGlobalFloatVariable(FIM_VID_ANGLE);
+			fim::string c=current();
+#ifdef FIM_AUTOCMDS
+//			autocmd_exec("PreRotate",c);//FIXME
+			autocmd_exec("PreScale",c);//FIXME
+#endif
+			if(c_image())
+			{
+				if(angle)
+					{if(image())image()->rotate((float)angle);}
+				else	
+					{if(image())image()->rotate();}
+			}
+#ifdef FIM_AUTOCMDS
+//			autocmd_exec("PostRotate",c);//FIXME
+			autocmd_exec("PostScale",c);//FIXME
+#endif
+		}
+		return "";
+	}
 
 	fim::string Browser::magnify(const args_t &args)
 	{
@@ -1151,7 +1259,9 @@ namespace fim
 		 */ 
 		if(c_image())
 		{
-			float factor = (float)getGlobalFloatVariable("magnify_factor");
+			float factor;
+			factor = firstforzero(args);
+			if(!factor) factor = (float)getGlobalFloatVariable(FIM_VID_MAGNIFY_FACTOR);
 			fim::string c=current();
 #ifdef FIM_AUTOCMDS
 			autocmd_exec("PreScale",c);
@@ -1159,9 +1269,15 @@ namespace fim
 			if(c_image())
 			{
 				if(factor)
-					{if(image())image()->magnify(factor);}
+					{
+						if(image())image()->magnify(factor);
+						if(viewport())viewport()->scale_position_magnify(factor);
+					}
 				else	
-					{if(image())image()->magnify();}
+					{
+						if(image())image()->magnify();
+						if(viewport())viewport()->scale_position_magnify();
+					}
 			}
 #ifdef FIM_AUTOCMDS
 			autocmd_exec("PostScale",c);
@@ -1177,15 +1293,25 @@ namespace fim
 		 */ 
 		if(c_image())
 		{
-			float factor = (float)getGlobalFloatVariable("reduce_factor");
+			float factor;
+			factor = firstforzero(args);
+			if(!factor) factor = (float)getGlobalFloatVariable(FIM_VID_REDUCE_FACTOR);
 			fim::string c=current();
 #ifdef FIM_AUTOCMDS
 			autocmd_exec("PreScale",c);
 #endif
 			if(c_image())
 			{
-				if(factor) {if(image())image()->reduce(factor);}
-				else	{if(image())image()->reduce();}
+				if(factor)
+					{
+						if(image())image()->reduce(factor);
+						if(viewport())viewport()->scale_position_reduce(factor);
+					}
+				else	
+					{
+						if(image())image()->reduce();
+						if(viewport())viewport()->scale_position_reduce();
+					}
 			}
 #ifdef FIM_AUTOCMDS
 			autocmd_exec("PostScale",c);
@@ -1243,15 +1369,10 @@ namespace fim
 		/*
 		 *	a const pointer to the currently loaded image
 		 */
-	#ifdef FIM_WINDOWS
-		if( cc.current_viewport() )
-			return cc.current_viewport()->c_getImage();
+		if( commandConsole.current_viewport() )
+			return commandConsole.current_viewport()->c_getImage();
 		else
 			return NULL;
-	#else
-		if(!only_viewport)return NULL;
-		return only_viewport->c_getImage();
-	#endif
 	}
 
 	Image *Browser::image()const
@@ -1259,15 +1380,10 @@ namespace fim
 		/*
 		 *	the image loaded in the current viewport is returned
 		 */
-	#ifdef FIM_WINDOWS
-		if( cc.current_viewport() )
-			return cc.current_viewport()->getImage();
+		if( commandConsole.current_viewport() )
+			return commandConsole.current_viewport()->getImage();
 		else
 			return NULL;
-	#else
-		if(!only_viewport)return NULL;
-		return only_viewport->getImage();
-	#endif
 	}
 
 	Viewport* Browser::viewport()const
@@ -1278,12 +1394,7 @@ namespace fim
 		 *
 		 * NULL is returned in case no viewport is loaded.
 		 * */
-#ifdef FIM_WINDOWS
-
-		return (cc.current_viewport());
-#else
-		return only_viewport;
-#endif
+		return (commandConsole.current_viewport());
 	}
 
 	fim::string Browser::current()const
@@ -1325,11 +1436,85 @@ namespace fim
 		/*
 		 *	pushes a new image filename on the back of the image list
 		 */
-		for(unsigned int i=0;i<args.size();++i)
+		for(size_t i=0;i<args.size();++i)
+		{
+#ifdef FIM_SMART_COMPLETION
+			/* due to this patch, filenames could arrive here with some trailing space. 
+			 * we trim them here, which is not correct if someone intends to push
+			 * a space-trailing filenme.
+			 * 
+			 * FIXME : regard this as a bug
+			 * */
+			fim::string ss=args[i];
+			ss.substitute(" +$","");
+			push(ss);
+#else
 			push(args[i]);
+#endif
+		}
 		return "";
 	}
 
+	fim::string Browser::next_page(const args_t &args)
+	{
+		/*
+		 * jumps one page forward in the current multipage image
+		 */
+		fim::string c=current();
+#ifdef FIM_AUTOCMDS
+		autocmd_exec("PreNext",c);
+#endif
+		if(c_image())
+		{
+			if(image())image()->next_page();
+		}
+#ifdef FIM_AUTOCMDS
+		autocmd_exec("PostNext",c);
+#endif
+		return "";
+	}
+
+	fim::string Browser::prev_page(const args_t &args)
+	{
+		/*
+		 * jumps one page backward in the current multipage image
+		 */
+		fim::string c=current();
+#ifdef FIM_AUTOCMDS
+		autocmd_exec("PreNext",c);
+#endif
+		if(c_image())
+		{
+			if(image())image()->prev_page();
+		}
+#ifdef FIM_AUTOCMDS
+		autocmd_exec("PostNext",c);
+#endif
+		return "";
+	}
+
+	fim::string Browser::next_picture(const args_t &args)
+	{
+		/*
+		 * FIX ME
+		 * */
+		if(c_image() && c_image()->have_nextpage())
+			return next_page(args);
+		else
+			return next(args.size()>0?((int)args[0]):1);
+	}
+
+	fim::string Browser::prev_picture(const args_t &args)
+	{
+		/*
+		 * FIX ME
+		 * */
+		if(c_image() && c_image()->have_prevpage())
+			return prev_page(args);
+		else
+			return prev(args.size()>0?((int)args[0]):1);
+	}
+
 	fim::string Browser::next(const args_t &args)
 	{
 		/*
